# -*- encoding: utf-8 -*-
#オリジナルライセンス
require 'spec_helper'

describe OriginalLicensesController do
  before do
    @admin = Factory :admin
    @lc = Factory :license
    @user = Factory( :user_yas)
    @author = @user.author
    @artist = Factory :artist_yas, :author_id => @author.id
  end
  describe '一覧表示に於いて' do
    before do
      sign_in @user
      @ol = Factory :original_license, :license_id => @lc.id
      OriginalLicense.stub(:list).and_return([@ol, @ol, @ol])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq OriginalLicense.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq OriginalLicense.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq OriginalLicense.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it 'ライセンスモデルに一覧を問い合わせている' do
        OriginalLicense.should_receive(:list).exactly(1)
        get :index
      end
      it '@original_licensesにリストを取得している' do
        get :index
        assigns(:original_licenses).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはライセンスっぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("url").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 okを返す' do
        get :index
        response.status.should eq 200
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      sign_in @user
      @ol = Factory :original_license, :license_id => @lc.id
      OriginalLicense.stub(:show).and_return(@ol)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @ol.id
        response.should be_success
      end
      it 'ライセンスモデルに単体取得を問い合わせている' do
        OriginalLicense.should_receive(:show).exactly(1)
        get :show
      end
      it '@original_licenseにアレを取得している' do
        get :show, :id => @ol.id
        assigns(:original_license).id.should eq(@ol.id)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @ol.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @ol.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          get :show, :id => @ol.id, :format => :json
          json = JSON.parse response.body
          json["name"].should match(/peta/)
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード200 okを返す' do
        get :show, :id => @ol.id
        response.status.should eq 200
      end
    end
  end
  
  describe '新規作成フォーム表示に於いて' do
    before do
      sign_in @admin
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :new
        response.should be_success 
      end
      it '@original_licenseに新規データを用意している' do
        get :new
        assigns(:original_license).should be_a_new(OriginalLicense)
      end
      it 'オリジナルライセンスモデルにデフォルト値補充を依頼している' do
        OriginalLicense.any_instance.should_receive(:supply_default).exactly(1)
        get :new
      end
      context 'html形式' do
        it 'newテンプレートを描画する' do
          get :new
          response.should render_template("new")
        end
      end
      context 'js形式' do
        it 'new.jsテンプレートを描画する' do
          get :new, :format => :js
          response.should render_template("new")
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :new
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :new
          response.body.should redirect_to '/admins/sign_in'
        end
      end
      context 'js形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :new, :format => :js
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :new, :format => :js
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '新規作成に於いて' do
    before do
      sign_in @admin
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'モデルに保存依頼する' do
        OriginalLicense.any_instance.should_receive(:store).exactly(1)
        post :create, :original_license => Factory.attributes_for(:original_license)
      end
      it "@original_licenseに作成されたオリジナルライセンスを保持していて、それがDBにある" do
        post :create, :original_license => Factory.attributes_for(:original_license)
        assigns(:original_license).should be_a(OriginalLicense)
        assigns(:original_license).should be_persisted
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          OriginalLicense.any_instance.stub(:store).and_return(true)
          post :create, :original_license => Factory.attributes_for(:original_license)
          response.status.should eq 302
        end
        it '作成されたオリジナルライセンスの表示ページへ遷移する' do
#          OriginalLicense.any_instance.stub(:save).and_return(true)
          post :create, :original_license => Factory.attributes_for(:original_license)
          response.should redirect_to(OriginalLicense.last)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
#          OriginalLicense.any_instance.stub(:save).and_return(true)
          post :create, :original_license => Factory.attributes_for(:original_license), :format => :json
          response.should be_success 
        end
        it '作成されたオリジナルライセンスをjsonデータで返す' do
          post :create, :original_license => Factory.attributes_for(:original_license), :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          post :create, :original_license => Factory.attributes_for(:original_license), :format => :json
          json = JSON.parse response.body
          json["name"].should match(/peta/)
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :create, :original_license => Factory.attributes_for(:original_license)
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          post :create, :original_license => Factory.attributes_for(:original_license)
          response.body.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          post :create, :original_license => Factory.attributes_for(:original_license), :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          post :create, :original_license => Factory.attributes_for(:original_license), :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '検証、保存に失敗した' do
      before do
        OriginalLicense.any_instance.stub(:save).and_return(false)
      end
      it "未保存のオリジナルライセンスを保持している" do
        post :create, :original_license => {}
        assigns(:original_license).should be_a_new(OriginalLicense)
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          post :create, :original_license => {}
          response.status.should eq 200
        end
        it '新規ページを描画する' do
          post :create, :original_license => {}
          response.should render_template("new")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          post :create, :original_license => {}, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          post :create, :original_license => {}, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

  describe '編集フォーム表示に於いて' do
    before do
      @ol = Factory :original_license, :license_id => @lc.id
      sign_in @admin
      sign_in @user
      OriginalLicense.stub(:show).and_return(@ol)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :edit, :id => @ol.id
        response.should be_success 
      end
      it 'オリジナルライセンスモデルに単体取得を問い合わせている' do
        OriginalLicense.should_receive(:show).exactly(1)
        get :edit, :id => @ol.id
      end
      it '@original_licenseにデータを用意している' do
        get :edit, :id => @ol.id
        assigns(:original_license).should eq @ol
      end
      context 'html形式' do
        it 'editテンプレートを描画する' do
          get :edit, :id => @ol.id
          response.should render_template("edit")
        end
      end
      context 'js形式' do
        it 'edit.jsテンプレートを描画する' do
          get :edit, :id => @ol.id, :format => :js
          response.should render_template("edit")
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :edit, :id => @ol.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :edit, :id => @ol.id
          response.body.should redirect_to '/admins/sign_in'
        end
      end
      context 'js形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :edit, :id => @ol.id, :format => :js
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :edit, :id => @ol.id, :format => :js
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '更新に於いて' do
    before do
      @ol = Factory :original_license, :license_id => @lc.id
      sign_in @admin
      sign_in @user
      @attr = Factory.attributes_for(:original_license, :name => 'new lc')
    end
    context '事前チェックしておく' do
      it 'オリジナルライセンスモデルに単体取得を問い合わせている' do
        OriginalLicense.stub(:show).with(any_args()).and_return @ol
        OriginalLicense.should_receive(:show).exactly(1)
        put :update, :id => @ol.id, :original_license => @attr
      end
      it 'モデルに更新を依頼する' do
        OriginalLicense.any_instance.should_receive(:store).with(any_args)
        put :update, :id => @ol.id, :original_license => @attr
      end
      it '@original_licenseにアレを取得している' do
        put :update, :id => @ol.id, :original_license => @attr
        assigns(:original_license).id.should eq(@ol.id)
      end
    end
    context 'つつがなく終わるとき' do
      it '更新される' do
        put :update, :id => @ol.id, :original_license => @attr
        OriginalLicense.find(@ol.id).name.should eq 'new lc'
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          OriginalLicense.any_instance.stub(:store).with(any_args()).and_return(true)
          put :update, :id => @ol.id, :original_license => @attr
          response.status.should eq 302
        end
        it '更新されたオリジナルライセンスの表示ページへ遷移する' do
          put :update, :id => @ol.id, :original_license => @attr
          response.should redirect_to(@ol)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          OriginalLicense.any_instance.stub(:store).with(any_args()).and_return(true)
          put :update, :id => @ol.id, :original_license => @attr, :format => :json
          response.should be_success 
        end
        it 'ページ本体は特に返さない' do
          OriginalLicense.any_instance.stub(:store).with(any_args()).and_return(true)
          put :update, :id => @ol.id, :original_license => @attr, :format => :json
          response.body.should match /./
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      it 'ステータスコード302 Foundを返す' do
        put :update, :id => @ol.id, :original_license => @attr
        response.status.should eq 302
      end
      context 'html形式' do
        it 'サインインページへ遷移する' do
          put :update, :id => @ol.id, :original_license => @attr
          response.body.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
        it '応答メッセージにUnauthorizedを返す' do
          put :update, :id => @ol.id, :original_license => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '検証、保存に失敗したとき' do
      before do
        OriginalLicense.any_instance.stub(:store).and_return(false)
      end
      context 'html形式' do
        it 'ステータスコード200 Okを返す' do
          put :update, :id => @ol.id, :original_license => @attr
          response.status.should eq 200
        end
        it '編集ページを描画する' do
          put :update, :id => @ol.id, :original_license => @attr
          response.should render_template("edit")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          OriginalLicense.any_instance.stub(:store).and_return(false)
          put :update, :id => @ol.id, :original_license => @attr, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          put :update, :id => @ol.id, :original_license => @attr, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

  describe 'インポートに於いて' do
    before do
      sign_in @admin
      sign_in @user
      #テストデータを用意してね
      @f = Rails.root + 'spec/json/original_license.json'
      @t = File.open(@f, 'r').read
      @j = JSON.parse @t
      @fs = Rails.root + 'spec/json/original_licenses.json'
      @ts = File.open(@fs, 'r').read
      @js = JSON.parse @ts
      @fes = Rails.root + 'spec/json/invalid_original_licenses.json'
      @tes = File.open(@fes, 'r').read
      @jes = JSON.parse @tes
    end
    context '事前チェックしておく' do
      before do
        #異常な行を返すから、正常の意味で空を返す
        OriginalLicense.stub(:import).with(any_args()).and_return([])
      end
      it "@dataに渡したデータを保持している" do
        post :import, :file => @t
        assigns(:data).should_not be_nil
      end
      it 'モデルにインポート依頼する' do
        OriginalLicense.should_receive(:import).with(any_args()).exactly(1)
        post :import, :file => @t
      end
      it "@errorsに結果を保持している" do
        post :import, :file => @t
        assigns(:errors).should eq []
      end
    end
    context 'つつがなく終わるとき' do
      before do
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :import, :file => @t
          response.status.should eq 302
        end
        it '管理者向けオリジナルライセンス一覧ページへ遷移する' do
          post :import, :file => @t
          response.should redirect_to('/original_licenses/list')
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          post :import, :file => @t, :format => :json
          response.should be_success 
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :import, :file => @t
          response.status.should eq 302
        end
        it '管理者サインインページへ遷移する' do
          post :import, :file => @t
          response.body.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          post :import, :file => @t, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          post :import, :file => @t, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '作家権限がないとき' do
      #必要なのは管理者権限であって作家権限ではない。成功を見届ける
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :import, :file => @t
          response.status.should eq 302
        end
        it '管理者向けオリジナルライセンス一覧ページへ遷移する' do
          post :import, :file => @t
          response.should redirect_to('/original_licenses/list')
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          post :import, :file => @t, :format => :json
          response.should be_success 
        end
      end
    end
    context '検証、保存に失敗した' do
      before do
        #異常な行を返す
        OriginalLicense.stub(:import).with(any_args()).and_return(
          [OriginalLicense.new(Factory.attributes_for(:original_license))]
        )
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          post :import, :file => @t
          response.status.should eq 200
        end
        it 'resultページを描画する' do
          post :import, :file => @t
          response.should render_template("result")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          post :import, :file => @t, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          post :import, :file => @t, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end
  
end
